module TERASIC_LTC2308(
	input							clk, // max 40MHz
	input							reset_n,
	
	// avmm
	input							slave_chipselect_n,
	input	[3:0]					slave_address,
	input							slave_write,
	input	[31:0]				slave_writedata,
	input							slave_read,
	output	reg	[31:0]	slave_readdata,
	
	// spi
	output						ADC_CONVST,
	output	      			ADC_SCK,
	output		   			ADC_SDI,
	input 		   			ADC_SDO	
);

parameter CLOCK_DUR = 25;

////////////////////////////
// AVMM

// reg
// 0: control & status regsiter
// 1: adc channel 0
// 2: adc channel 1
// ...
// 8: adc channel 7



always @(posedge clk or negedge reset_n)
begin
	if (~reset_n)
		data_capture <= 1'b0;
	else if (~slave_chipselect_n  & slave_write)
	begin
		if (slave_address == 0)
			data_capture <= slave_writedata[0];
	end
	else if (~slave_chipselect_n  & slave_read)
	begin
		case(slave_address)
			0: slave_readdata <= {31'h00000000,data_ready};
			1: slave_readdata <= {20'h00000, data0};
			2: slave_readdata <= {20'h00000, data1};
			3: slave_readdata <= {20'h00000, data2};
			4: slave_readdata <= {20'h00000, data3};
			5: slave_readdata <= {20'h00000, data4};
			6: slave_readdata <= {20'h00000, data5};
			7: slave_readdata <= {20'h00000, data6};
			8: slave_readdata <= {20'h00000, data7};
		endcase
	end
end



/////////////////////////////

reg  data_capture;
wire data_ready;


wire [11:0]	data0;
wire [11:0]	data1;
wire [11:0]	data2;
wire [11:0]	data3;
wire [11:0]	data4;
wire [11:0]	data5;
wire [11:0]	data6;
wire [11:0]	data7;

defparam  LTC2308_ADC.CLOCK_DUR = CLOCK_DUR;

LTC2308 LTC2308_ADC(
	.clk(clk), // max 40mhz
	.reset_n(reset_n),
	
	//
	.data_capture(data_capture), // rise edge to trigger
	.data_ready(data_ready),
	.data0(data0),
	.data1(data1),
	.data2(data2),
	.data3(data3),
	.data4(data4),
	.data5(data5),
	.data6(data6),
	.data7(data7),
	
	// spi 
	.ADC_CONVST(ADC_CONVST),
	.ADC_SCK(ADC_SCK),
	.ADC_SDI(ADC_SDI),
	.ADC_SDO(ADC_SDO)

);



endmodule


