/*
 * Sensor.cpp
 *
 *  Created on: 2016/3/15
 *      Author: User
 */

#include "Sensor.h"
#include "terasic_includes.h"

#define ADC_START_BIT	0x01
#define ADC_DONE_BIT	0x01


#define REG_CONTROL_STATUS		0
#define REG_ADC_CHANNEL_BASE	1
#define POWER_SCALE	    (4)

CSensor::CSensor() {
	// TODO Auto-generated constructor stub
}

CSensor::~CSensor() {
	// TODO Auto-generated destructor stub
}


bool CSensor::ReadADC(alt_u16 szAdc[ADC_CHANNEL_NUM]){
	bool bSuccess = false, bTimeout = false, bDone = false;
	alt_u32 Status, TimeStart;
	int i;
	const  alt_u32 TimmeoutDur = alt_ticks_per_second()/10;

	// add IR ON/OFF control for reduce power consumption


	// start
	IOWR(ADC_LTC2308_BASE, REG_CONTROL_STATUS, 0x00);
	IOWR(ADC_LTC2308_BASE, REG_CONTROL_STATUS, 0x00 | ADC_START_BIT);
	IOWR(ADC_LTC2308_BASE, REG_CONTROL_STATUS, 0x00);

	// wait done
	TimeStart = alt_nticks();
	while(!bDone && !bTimeout){
		Status = IORD(ADC_LTC2308_BASE, REG_CONTROL_STATUS);
		if (Status & ADC_DONE_BIT)
			bDone = true; // finish
		else if ((alt_nticks() - TimeStart) >  TimmeoutDur)
			bTimeout = true; // timeout
	}

	// read data
	if (bDone & !bTimeout){
		bSuccess = true;
		for(i=0;i<ADC_CHANNEL_NUM;i++){
			szAdc[i] = IORD(ADC_LTC2308_BASE, i+REG_ADC_CHANNEL_BASE);
		}
	}



	return bSuccess;
}

bool CSensor::ReadLineSenor(alt_u16 szAdc[SENSOR_NUM]){
	bool bSuccess;
	alt_u16 szAdcAll[ADC_CHANNEL_NUM];
	int i;

	bSuccess = ReadADC(szAdcAll);
	if (bSuccess){
		for(i=0;i<SENSOR_NUM;i++)
			szAdc[SENSOR_NUM-i-1] = szAdcAll[i]; // exchange index
	}

	return bSuccess;

}


bool CSensor::ReadInputPower(alt_u16 *pmV){
	bool bSuccess;
	alt_u16 szAdcAll[ADC_CHANNEL_NUM];

	bSuccess = ReadADC(szAdcAll);
	if (bSuccess){
		*pmV = szAdcAll[ADC_CHANNEL_NUM-1] * POWER_SCALE;
	}

	return bSuccess;
}

bool CSensor::ReadLineSenorAndInputPower(alt_u16 szAdc[SENSOR_NUM], alt_u16 *pmV){
	bool bSuccess;
	alt_u16 szAdcAll[ADC_CHANNEL_NUM];
	int i;

	bSuccess = ReadADC(szAdcAll);
	if (bSuccess){
		for(i=0;i<SENSOR_NUM;i++)
			szAdc[SENSOR_NUM-i-1] = szAdcAll[i]; // exchange index
		*pmV = szAdcAll[ADC_CHANNEL_NUM-1] * POWER_SCALE;
	}

	return bSuccess;
}


void CSensor::DumpLineSenor(alt_u16 szAdc[SENSOR_NUM]){
	int i;

	printf("Adc:");
	for(i=0;i<SENSOR_NUM;i++){
		printf("%d", szAdc[i]);
		if (i+1 != SENSOR_NUM)
			printf(", ");
	}
	printf("\r\n");
}

void CSensor::DumpLineSenor(void){
	alt_u16 szAdc[SENSOR_NUM];


	ReadLineSenor(szAdc);
	DumpLineSenor(szAdc);
}




void CSensor::Test(void){
	bool bSuccess;
	alt_u16 szAdc[SENSOR_NUM], nMaxi = 0, nMini = 4000, mV;
	int i, nIndex=1;


	while(1){
		// read data
		bSuccess = ReadLineSenorAndInputPower(szAdc, &mV);
		if (!bSuccess){
			printf("failed to read adc!\r\n");
			continue;
		}

		// maxi & mini for line sensor
		for(i=0;i<SENSOR_NUM;i++){
			if (szAdc[i] > nMaxi)
				nMaxi = szAdc[i];
			else if (szAdc[i] < nMini)
				nMini = szAdc[i];
		}


		// dump data
		printf("[%d]",nIndex++);
		for(i=0;i<SENSOR_NUM;i++){
			printf("%d", (int)szAdc[i]);
			if (i+1 != SENSOR_NUM)
				printf(", ");
		}
		printf(" -- maxi=%d, mini=%d, vol=%dmV", (int)nMaxi, (int)nMini, mV);

		printf("\r\n");
		usleep(100*1000);


	} // while

}


void CSensor::Test2(void){
	bool bSuccess;
	alt_u16 szAdcAll[ADC_CHANNEL_NUM];
	int i, nIndex=1;


	while(1){
		// read data
		bSuccess = ReadADC(szAdcAll);
		if (!bSuccess){
			printf("failed to read adc!\r\n");
			continue;
		}

		// dump data
		printf("[%d]",nIndex++);
		for(i=0;i<ADC_CHANNEL_NUM;i++){
			printf("%d", (int)szAdcAll[i]);
			if (i+1 != ADC_CHANNEL_NUM)
				printf(", ");
		}
		printf("\r\n");
		usleep(100*1000);


	} // while

}

