
#include <stdio.h>
#include <math.h>
#include <stdarg.h>
#include <unistd.h>
#include <stdio.h>
#include "terasic_includes.h"
#include "Motor.h"
#include "Car.h"
#include "Sensor.h"
#include "IrRx.h"

// threshold for exception case
#define BLACK_THRESHOLD  2800
#define WHITE_THRESHOLD  700

#define TYPE_NEW

#define xSHOW_LOG


int myprintf(char *fmt, ...);

#ifdef SHOW_LOG
static char sprint_buf[1024];
int myprintf(char *fmt, ...)
{

    va_list args;
    int n;
    va_start(args, fmt);
    n = vsprintf(sprint_buf, fmt, args);
    va_end(args);
    write(1, sprint_buf, n);
    return n;
}
#else
int myprintf(char *fmt, ...){
	return 0;
}

#endif



void ShowPowerStatus(alt_u16 mV){
	alt_u8 LedMask;
	const int nBaseVol = 3800; // mV


	if (mV < nBaseVol)
		LedMask = 0x00;
	else
		LedMask = ((alt_u32)0x00FFFFFF << (int)((mV - nBaseVol)/100)) >> 24;

	//printf("mask=%xh\r\n", LedMask);

	IOWR(LED_BASE, 0x00, LedMask); // led is high-active


	myprintf("Input Voltage:%dmV\r\n", mV);

}


bool StartButton()
{
	bool bStart = false;
	alt_u32 Value;
	Value = IORD(KEY_BASE,0);
	//printf("Input Kye:%xh\r\n", Value);
	if (Value != 0x03)
		bStart = true;

	return bStart;
}

void ShowSensorStatus(alt_u16 szAdc[SENSOR_NUM])
{
	int i;
	alt_u32 LedMask=0x00, LedBitMask=0x80;

	for(i=0;i<SENSOR_NUM;i++){
		if (szAdc[i] >= 2000)
			LedMask |= LedBitMask;
		LedBitMask >>= 1;
	}
	IOWR(LED_BASE, 0x00, LedMask);

}



void LightOn(bool bEnableLeft, bool bEnableRight){
	IOWR(LIGHT_LEFT_BASE, 0x00, bEnableLeft?0x01:0x00);
	IOWR(LIGHT_RIGHT_BASE, 0x00, bEnableRight?0x01:0x00);
}


void BuzzerOn(bool bEnable){
	IOWR(BUZZER_BASE, 0x00, bEnable?0x01:0x00);
}

void Welcome(CCar &Car){
	int i;
	const int nSpeed = 10;

	Car.Stop();
	Car.SetSpeed(nSpeed);

	for(i=0;i<2;i++){
		// FF
		Car.SetSpeed(nSpeed);
		Car.Start();


		// light on
		LightOn(true, true);

		// beep off
		BuzzerOn(false);

		usleep(50*1000);
		Car.Stop();

		///////////////////////

		Car.SetSpeed(-nSpeed);
		Car.Start();

		// light off
		LightOn(false, false);

		// beep on
		BuzzerOn(true);

		usleep(50*1000);
		Car.Stop();

	}
	// beep off
	BuzzerOn(false);

}

bool IsBuzzerLightEnable(void){
	bool bEnable = false;

	// SW3 up to enable
	if (IORD(SWITCH_BASE,0x00) & 0x01)
		bEnable = true;
	return bEnable;
}


bool HandleIR(CCar &Car, CIrRx &IR){
	bool bStop = true;
	alt_u32 Command;
	static int nSpeed = 50;
	static bool bLedOn = true;
	static int nBeepOnCnt = 0;
	static int nLedOnCnt = 0;
	const int nMaxCnt = 1500;

	if (nBeepOnCnt > 0){
		nBeepOnCnt--;
		if (nBeepOnCnt == 0)
			BuzzerOn(false);
	}

	if (nLedOnCnt > 0){
		nLedOnCnt--;
		if (nLedOnCnt == 0)
			LightOn(false, false);
	}

	if (!IR.IsEmpty()){
		Command = IR.Pop();
		myprintf("command:%04xh\r\n", Command);
		switch(Command){
		case CIrRx::IR_PLAY_PAUSE:
			bStop = false;
			IR.Clear();
			BuzzerOn(false);
			break;
		case CIrRx::IR_CH_UP:
		case CIrRx::IR_VOL_UP:
			Car.SetSpeed(nSpeed);
			Car.Start();
			break;
		case CIrRx::IR_CH_DOWN:
		case CIrRx::IR_VOL_DOWN:
			Car.SetSpeed(-nSpeed);
			Car.Start();
			break;
		case CIrRx::IR_ADJ_RIGHT:
			Car.SetSpeed(nSpeed, 0);
			Car.Start();
			break;
		case CIrRx::IR_ADJ_LEFT:
			Car.SetSpeed(0, nSpeed);
			Car.Start();
			break;
		case CIrRx::IR_POWER:
			Car.Stop();
			break;
		case CIrRx::IR_NUM_0:
			nSpeed = 10;
			Car.SetSpeed(nSpeed, nSpeed);
			break;
		case CIrRx::IR_NUM_1:
			nSpeed = 20;
			Car.SetSpeed(nSpeed, nSpeed);
			break;
		case CIrRx::IR_NUM_2:
			nSpeed = 30;
			Car.SetSpeed(nSpeed, nSpeed);
			break;
		case CIrRx::IR_NUM_3:
			nSpeed = 40;
			Car.SetSpeed(nSpeed, nSpeed);
			break;
		case CIrRx::IR_NUM_4:
			nSpeed = 50;
			Car.SetSpeed(nSpeed, nSpeed);
			break;
		case CIrRx::IR_NUM_5:
			nSpeed = 60;
			Car.SetSpeed(nSpeed, nSpeed);
			break;
		case CIrRx::IR_NUM_6:
			nSpeed = 70;
			Car.SetSpeed(nSpeed, nSpeed);
			break;
		case CIrRx::IR_NUM_7:
			nSpeed = 80;
			Car.SetSpeed(nSpeed, nSpeed);
			break;
		case CIrRx::IR_NUM_8:
			nSpeed = 90;
			Car.SetSpeed(nSpeed, nSpeed);
			break;
		case CIrRx::IR_NUM_9:
			nSpeed = 100;
			Car.SetSpeed(nSpeed, nSpeed);
			break;
		case CIrRx::IR_NUM_A:
			BuzzerOn(true);
			nBeepOnCnt = nMaxCnt;
			break;
		case CIrRx::IR_NUM_B:
			LightOn(bLedOn, bLedOn);
			bLedOn = bLedOn?false:true;
			break;
		case CIrRx::IR_NUM_C:
			LightOn(true, true);
			BuzzerOn(true);
			nBeepOnCnt = nMaxCnt;
			nLedOnCnt = nMaxCnt;
			break;
		}
		IR.Clear();
	}

	return bStop;
}


void SensorBoardEnable(bool bEnable){
	IOWR(IR_LED_ON_BASE, 0x00, bEnable?0x01:0x00); // enable sensor board IR, high-active
}


int main()
{
	  bool bStop = FALSE, bBeepOn = false, bInvalid;
	  const float Speed_Low = 50;
	  const float Speed_Normal = 80;
	  const float Speed_High = 100;
	  float LeftSpeed, RightSpeed, Speed;

	  const float fWeight = 0.7;
	  float szAdc[SENSOR_NUM], fSum;
	  alt_u16 szAdcNow[SENSOR_NUM], mV;
	  char szError[256];

	  float kp = 1.0;
	  float ki = 0;
	  float kd = 8.0;

	  alt_u32 nTick = 1, nInterationInterval, nNextStartTime;
	  float error, intergral, derivative, last_error, output, turn;
	  int i, nInvalidCnt;
	  const int nMaxInvalidCnt = 10;

	  CCar 		Car(DC_MOTOR_LEFT_BASE, DC_MOTOR_RIGHT_BASE);
	  CSensor 	Sensor;
	  CIrRx     IR(IR_RX_BASE, IR_RX_IRQ, IR_RX_IRQ_INTERRUPT_CONTROLLER_ID);

	  printf("===== A-Cute Car Demo =====\r\n");
	  // enable hardware
	  IOWR(SCD_3V3_POWERON_N_BASE, 0x00, 0x00); // enable daughter card 3.3V, low-active
	  SensorBoardEnable(true);


	  Welcome(Car);

	  Car.Stop();

	  // show battery power
	  Sensor.ReadInputPower(&mV);
	  ShowPowerStatus(mV);
	  usleep(2*1000*1000); // wait 2 second

	  IR.Enable();
	  nInterationInterval = alt_ticks_per_second()/250;

	  printf("Go...\r\n");
      bStop = TRUE;
	  while(1){
		  if (bStop){
			  // turn of lcd & bee
	 		  LightOn(false, false);
	 		  BuzzerOn(false);

			  myprintf("Ready! Press any button to start.\n");
			  SensorBoardEnable(true);

			  while(bStop){

				  bStop = StartButton()?false:true;
				  if (!bStop)
					  continue; // abort while

				  // show status
				  Sensor.ReadLineSenor(szAdcNow);
				  ShowSensorStatus(szAdcNow);

				  // handle ir command
				  bStop = HandleIR(Car, IR);
			  } // while
			  myprintf("GO\n");

			  // init adc array
			  Sensor.ReadLineSenor(szAdcNow);
			  for(i=0;i<SENSOR_NUM;i++)
				  szAdc[i] = szAdcNow[i];

			  bStop = false;
			  last_error = 0;
			  intergral = 0;
			  derivative = 0;
			  nInvalidCnt = 0;

			  // update power statu to car
			  Sensor.ReadInputPower(&mV);
			  Car.SetInputPower(mV);
			  Car.Start();

			  // clear ir
			  IR.Clear();

			  nNextStartTime = alt_nticks() + nInterationInterval;


		  } // if stop
		  //Sensor.ReadLineSenor(szAdcNow);
		 // ShowSensorStatus(szAdcNow);

		  /////////////////////////////
		  // wait next sampling time
		  SensorBoardEnable(true);
		  if (alt_nticks() < nNextStartTime){
			  // wait
			  while(alt_nticks() < nNextStartTime);
			  nNextStartTime += nInterationInterval;
		  }else{
			  myprintf("sampling time is delayed!-------------\r\n");
			  nNextStartTime = alt_nticks() + nInterationInterval;
		  }

		  /////////////////////////////
		  //  read data and apply low pass filter
		  Sensor.ReadLineSenor(szAdcNow);
		  SensorBoardEnable(false); // power off for reduce power consumption
		  ShowSensorStatus(szAdcNow);

		  // low pass filter
		  for(i=0;i<SENSOR_NUM;i++){
			//  Adc[i] = AdcNow[i];
			  szAdc[i] = szAdc[i] * fWeight + (float)szAdcNow[i]*(1.0-fWeight);
		  }

		  /////////////////////////////
		  // stop check

		  // out of track
		  bInvalid = true;
		  for(i=0;i<SENSOR_NUM && bInvalid;i++){
			  if (szAdc[i] > WHITE_THRESHOLD)
				  bInvalid = false;
		  } // for
		  if (bInvalid)
			  strcpy(szError, "smart car can not see line");

		  // in air
		  if (!bInvalid){
			  bInvalid = true;
			  for(i=0;i<SENSOR_NUM && bInvalid;i++){
				  if (szAdc[i] < BLACK_THRESHOLD)
					  bInvalid = false;
			  } // for
			  if (bInvalid)
				  strcpy(szError, "smart car in air");
		  }

		  // handle invalid condition
		  if (bInvalid){
			  if (nInvalidCnt++ > nMaxInvalidCnt)
				  bStop = true;
		  }else{
			  nInvalidCnt = 0;
		  }

		  // remote stop
		  if (!bStop){
			  if (!IR.IsEmpty()){
				  bStop = true;
				  strcpy(szError, "ir abort");
			  }
		  }



		  /////////////////////////////////////////
		  // calculate error
#ifdef SHOW_LOG
		  Sensor.DumpLineSenor(szAdcNow);
#endif

		  error = 0.0;
		  for(i=0;i<SENSOR_NUM;i++)
			  error += szAdc[i] * (i+1);

		  fSum = 0;
		  for(i=0;i<SENSOR_NUM;i++)
			  fSum += szAdc[i];
		  if (fSum > 0){
			  error /= fSum;
			  error -= 4.0;  // mean is 4.0
		  }else{
			  error = 0;
			 // bStop = true;
			 // strcpy(szError, "sensor invalid input");
		  }

		  ///////////////////////////////////
		  // PID update
		 intergral = intergral + error;
		 derivative = error - last_error;
		 last_error = error;
		 output = (kp * error + ki * intergral + kd * derivative); // PID
		 turn = output * 100.0;

		 if (turn >=50 )
			 Speed = Speed_Low;
		 else if (turn >= 20)
			 Speed = Speed_Normal;
		 else
			 Speed = Speed_High;

		 LeftSpeed = Speed + turn;
		 RightSpeed = Speed - turn;

		 // buzzer & light control
		 bBeepOn = ((fabs(turn) >= 60.0) && IsBuzzerLightEnable())?true:false;
 		 LightOn((turn < 0)?bBeepOn:false, (turn > 0)? bBeepOn: false);
 		 BuzzerOn(bBeepOn);


 		 //////////////////////////////////////
		 // apply PID result to motor
		 if(bStop){
			 Car.Stop();
			 myprintf("stop: %s..................\r\n", szError);
#ifdef SHOW_LOG
			 Sensor.DumpLineSenor(szAdcNow);
#endif //SHOW_LOG
		 }else{
			 Car.SetSpeed(LeftSpeed, RightSpeed);
		 }


		 if (nTick == 1)
			 myprintf("%s\t%s\t%s\t%s\t%s\r\n", "nTick", "error", "turn", "LeftSpeed", "RightSpeed");
		 myprintf("[%d]%.3f\t%.3f\t%.1f\t%.1f\r\n", nTick, error, turn, LeftSpeed, RightSpeed);
		 nTick++;
	  }
  printf("done");
  return 0;
}
