module TERASIC_PWM_SCD(
	clk,
	reset_n,
	//
	s_cs,
	s_address,
	s_write,
	s_writedata,
	s_read,
	s_readdata,
	
	//
	PWM
);

`define REG_TOTAL_DUR 0
`define REG_HIGH_DUR	 1
`define REG_ABORT     2

input 					 clk;
input						 reset_n;

input						 s_cs;
input				[1:0]	 s_address;
input						 s_read;
output	reg	[31:0] s_readdata;
input						 s_write;
input				[31:0] s_writedata;

output	reg			 PWM;

reg				[31:0] total_dur;
reg				[31:0] high_dur;
reg	      	[31:0] tick;
reg 						 abort;



always @ (posedge clk or negedge reset_n)
begin
	if (~reset_n)
	begin
		high_dur <= 0;
		total_dur <= 0;
		abort <= 0;
	end
	else if (s_cs & s_write)
	begin
	  case(s_address)
	  `REG_TOTAL_DUR: total_dur<= s_writedata;
	  `REG_HIGH_DUR: 	high_dur <= s_writedata;
	  `REG_ABORT:     abort 	<= s_writedata;
	  default:high_dur <= high_dur;
	  endcase 
	end
	
	else if (s_cs & s_read)
	begin
	  case(s_address)
	  `REG_TOTAL_DUR: s_readdata <= total_dur;
	  `REG_HIGH_DUR: 	s_readdata <= high_dur;
	  `REG_ABORT:     s_readdata <= abort;
	  default:s_readdata <= s_readdata;
	  endcase 
	end
		
end


reg AbortTrigger;
reg AbortEnable;

always @ (posedge clk or negedge reset_n)
begin
	if (~reset_n)
	begin
		tick <= 0;
	end
	else if (tick >= total_dur)
	begin
	  tick <= 0;
	  AbortTrigger = ~AbortTrigger;
	end
	else
		tick <= tick + 1;
end

always@(AbortTrigger)
begin
  if(abort)
    AbortEnable <= 1;
  else
    AbortEnable <= 0;
end


always @ (posedge clk or negedge reset_n)
begin
	if (~reset_n)
		PWM <= 0;
	else
		PWM <= (tick < high_dur && AbortEnable == 0)?1'b1:1'b0;//duck die width(5us)250  
end


endmodule
